#ifndef SWUART_H
#define SWUART_H

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>

#define TICKS2WAITONE       103  //!< Wait one bit period.
#define TICKS2WAITONE_HALF  155  //!< Wait one and a half bit period.

#define INTERRUPT_EXEC_CYCL   9       //!< Cycles to execute interrupt rutine from interrupt.

#define TX_PIN           PB4                //!< Transmit data pin
#define RX_PIN           PB3                //!< Receive data pin
#define RX_PCINT		 PCINT3				//!< Receive PCINT
#define TCCR             TCCR0A             //!< Timer/Counter Control Register
#define TCCR_P           TCCR0B             //!< Timer/Counter Control (Prescaler) Register
#define OCR              OCR0A              //!< Output Compare Register
#define EXT_IFR          GIFR               //!< External Interrupt Flag Register
#define EXT_ICR          MCUCR              //!< External Interrupt Control Register
#define TRXDDR  		 DDRB				//!< DDR-Port fr den UART
#define TRXPORT 		 PORTB				//!< Ausgangsport fr den UART
#define TRXPIN  		 PINB				//!< Eingangsport fr den UART


#define ENABLE_TIMER_INTERRUPT( )       ( TIMSK |= ( 1<< OCIE0A ) )
#define DISABLE_TIMER_INTERRUPT( )      ( TIMSK &= ~( 1<< OCIE0A ) )
#define CLEAR_TIMER_INTERRUPT( )        ( TIFR  = ((1 << OCF0A) ) )
#define ENABLE_EXTERNAL0_INTERRUPT( )   ( GIMSK |= ( 1<< PCIE ) )
#define DISABLE_EXTERNAL0_INTERRUPT( )  while (! (TRXPIN & ( 1 << RX_PIN )) ); ( GIMSK &= ~( 1<< PCIE ) )
#define CLEAR_EXTERNAL0_INTERRUPT( )     

#define SET_TX_PIN( )    ( TRXPORT |= ( 1 << TX_PIN ) )
#define CLEAR_TX_PIN( )  ( TRXPORT &= ~( 1 << TX_PIN ) )
#define GET_RX_PIN( )    ( TRXPIN & ( 1 << RX_PIN ) )


typedef enum
{
    IDLE,                                       //!< Idle state, both transmit and receive possible.
	TRANSMIT_START_BIT,							// Startbit senden
    TRANSMIT,                                   //!< Transmitting byte.
    TRANSMIT_STOP_BIT1,                          //!< Transmitting stop bit.
    TRANSMIT_STOP_BIT2,                          //!< Transmitting stop bit.
	TRANSMIT_END,
	RECEIVE_FIRST,
    RECEIVE,                                    //!< Receiving byte.
    DATA_PENDING                                //!< Byte received and ready to read.

}AsynchronousStates_t;


void swuart_init();

void swuart_putchar(uint8_t c);
void swuart_print( const unsigned char *data );

uint8_t swuart_getchar();

AsynchronousStates_t swuart_getstate  ();

#endif
